/*
 * Copyright 2013-2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.cloud.zookeeper.discovery.watcher;

import java.util.ArrayList;
import java.util.List;

import org.apache.curator.x.discovery.ServiceDiscovery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cloud.zookeeper.ConditionalOnZookeeperEnabled;
import org.springframework.cloud.zookeeper.discovery.ZookeeperInstance;
import org.springframework.cloud.zookeeper.discovery.ZookeeperServiceDiscovery;
import org.springframework.cloud.zookeeper.discovery.dependency.ConditionalOnDependenciesPassed;
import org.springframework.cloud.zookeeper.discovery.dependency.ZookeeperDependencies;
import org.springframework.cloud.zookeeper.discovery.dependency.ZookeeperDependenciesAutoConfiguration;
import org.springframework.cloud.zookeeper.discovery.watcher.presence.DefaultDependencyPresenceOnStartupVerifier;
import org.springframework.cloud.zookeeper.discovery.watcher.presence.DependencyPresenceOnStartupVerifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * Provides hooks for observing dependency lifecycle in Zookeeper.
 * Needs custom dependencies to be set in order to work.
 *
 * @author Marcin Grzejszczak
 * @since 1.0.0
 *
 * @see ZookeeperDependencies
 */
@Configuration
@EnableConfigurationProperties
@ConditionalOnZookeeperEnabled
@ConditionalOnDependenciesPassed
@AutoConfigureAfter(ZookeeperDependenciesAutoConfiguration.class)
public class DependencyWatcherAutoConfiguration {

	@Autowired(required = false)
	private List<DependencyWatcherListener> dependencyWatcherListeners = new ArrayList<>();

	@Bean
	@ConditionalOnMissingBean
	public DependencyPresenceOnStartupVerifier dependencyPresenceOnStartupVerifier() {
		return new DefaultDependencyPresenceOnStartupVerifier();
	}

	@Bean(destroyMethod = "clearDependencyRegistrationHooks")
	@ConditionalOnBean(ZookeeperServiceDiscovery.class)
	@ConditionalOnMissingBean
	public DependencyRegistrationHookProvider dependencyWatcherDeprecated(
			ZookeeperServiceDiscovery serviceDiscovery,
			DependencyPresenceOnStartupVerifier dependencyPresenceOnStartupVerifier,
			ZookeeperDependencies zookeeperDependencies) {
		return new DefaultDependencyWatcher(serviceDiscovery,
				dependencyPresenceOnStartupVerifier,
				this.dependencyWatcherListeners,
				zookeeperDependencies);
	}

	@Bean(destroyMethod = "clearDependencyRegistrationHooks")
	@ConditionalOnMissingBean({ DependencyRegistrationHookProvider.class, ZookeeperServiceDiscovery.class })
	public DependencyRegistrationHookProvider dependencyWatcher(
			ServiceDiscovery<ZookeeperInstance> serviceDiscovery,
			DependencyPresenceOnStartupVerifier dependencyPresenceOnStartupVerifier,
			ZookeeperDependencies zookeeperDependencies) {
		return new DefaultDependencyWatcher(serviceDiscovery,
				dependencyPresenceOnStartupVerifier,
				this.dependencyWatcherListeners,
				zookeeperDependencies);
	}
}
